#include "net_load.hpp"

using namespace std;
using boost::property_tree::ptree;

net_load::net_load(DDSDomainParticipant *participant) : participant_ (participant)
{
    string sysconf_dir(SYSCONFDIR);
    qos_file_ += sysconf_dir + "qos/net_load_subscriber_qos.xml";
    cout << qos_file_ << endl;
    
    subscriber_ = participant_->create_subscriber(
        DDS_SUBSCRIBER_QOS_DEFAULT, NULL /* listener */, DDS_STATUS_MASK_NONE);

	if (subscriber_ == NULL) {
		printf("create_subscriber error\n");
	}

	
	/* Register type before creating topic */
	type_name_ = netloadTypeSupport::get_type_name();
	retcode_ = netloadTypeSupport::register_type(
		participant, type_name_);
		
	if (retcode_ != DDS_RETCODE_OK) {
		printf("register_type error %d\n", retcode_);
	}  
	
	topic_ = participant_->create_topic(
		"net_load",
        	type_name_, DDS_TOPIC_QOS_DEFAULT, NULL /* listener */,
	        DDS_STATUS_MASK_NONE);
	if (topic_ == NULL) {
	        printf("create_topic error\n");
	}

}

void net_load::receive_information(void) 
{
	/* Create a data reader listener */
	reader_listener_ = new netloadListener();

	/* To customize the data reader QoS, use
       the configuration file USER_QOS_PROFILES.xml */
	reader_ = subscriber_->create_datareader(
			topic_, DDS_DATAREADER_QOS_DEFAULT, reader_listener_,
			DDS_STATUS_MASK_ALL);

	if (reader_ == NULL) {
		printf("create_datareader error\n");
		delete reader_listener_;
	}


}

netloadListener::netloadListener()
{

    string sysconf_dir(SYSCONFDIR);
    string filename(sysconf_dir + "cavecanem_subscriber.xml");
    
	try {
		ptree pt;
		read_xml(filename,pt);
    	//Reading database data
        database_server_ = pt.get<string>("database.server");
    	database_name_ = pt.get<string>("database.name");
    	database_user_ = pt.get<string>("database.user");
    	database_password_ = pt.get<string>("database.password");
    	rows_ = pt.get<int>("plugins.net_load_plugin.max_rows");
    }
    
	catch(std::exception &e) {
		cerr << e.what() << endl;
		rows_ = 10;     
	}
    
    conn_.connect(
	    database_name_.c_str(), 
	    database_server_.c_str(), 
	    database_user_.c_str(),
	    database_password_.c_str());
	    
	mysqlpp::Query query = conn_.query();

    query << "CREATE TABLE IF NOT EXISTS hosts (" << 
             "hostname VARCHAR(30) NOT NULL PRIMARY KEY," <<
             "sysname VARCHAR(30)," <<
             "release_name VARCHAR(30)," <<
             "machine VARCHAR(30))";
    query.execute();	

	query.exec("DROP TABLE IF EXISTS net_load");

	query << "CREATE TABLE net_load (" <<
			"rrd_nl_seq_id INT UNSIGNED NOT NULL DEFAULT 0," <<
			"hostname VARCHAR(30) NOT NULL," <<
			"ts TIMESTAMP NOT NULL," <<
			"device VARCHAR(20) NOT NULL," <<
			"flags INT," <<
			"if_flags INT," <<
			"mtu INT," <<
			"net_addr VARCHAR(30)," <<
			"subnet_addr VARCHAR(30)," <<
			"hwaddress VARCHAR(30)," <<            			
			"packets_in INT UNSIGNED," <<
			"packets_out INT UNSIGNED," <<
			"bytes_in INT UNSIGNED," <<
			"bytes_out INT UNSIGNED," <<
			"bytes_total INT UNSIGNED," <<
			"errors_in INT UNSIGNED," <<
			"errors_out INT UNSIGNED," <<
			"errors_total INT UNSIGNED," <<
			"collisions INT UNSIGNED," <<			
			"PRIMARY KEY(rrd_nl_seq_id,hostname,device)," <<
			"FOREIGN KEY(hostname) REFERENCES hosts(hostname))";
	query.execute();
}

void netloadListener::on_data_available(DDSDataReader* reader)
{
    netloadDataReader *netload_reader = NULL;
    netloadSeq data_seq;
    DDS_SampleInfoSeq info_seq;
    DDS_ReturnCode_t retcode;
    int i;

    netload_reader = netloadDataReader::narrow(reader);
    if (netload_reader == NULL) {
        printf("DataReader narrow error\n");
        return;
    }

    retcode = netload_reader->take(
        data_seq, info_seq, DDS_LENGTH_UNLIMITED,
        DDS_ANY_SAMPLE_STATE, DDS_ANY_VIEW_STATE, DDS_ANY_INSTANCE_STATE);

    if (retcode == DDS_RETCODE_NO_DATA) {
        return;
    } else if (retcode != DDS_RETCODE_OK) {
        printf("take error %d\n", retcode);
        return;
    }

    for (i = 0; i < data_seq.length(); ++i) {
        
        mysqlpp::Query query = conn_.query();
        
        map<char *,host_info>::iterator hostinfo_map_it;
        hostinfo_map_it = hostinfo_map_.find(data_seq[i].host.hostname);
        //Dealing with hosts table
        if(hostinfo_map_it == hostinfo_map_.end()) {
            hostinfo_map_[data_seq[i].host.hostname] = data_seq[i].host;
		    query << "REPLACE INTO hosts VALUES (" <<
		            "'" << data_seq[i].host.hostname << "'," << 
		            "'" << data_seq[i].host.sysname << "'," <<
		            "'" << data_seq[i].host.release_name << "'," <<
		            "'" << data_seq[i].host.machine << "')";
            query.execute();
        }    
    
        if (info_seq[i].valid_data) {
        	string aux = data_seq[i].host.hostname;
        	aux += data_seq[i].device;
			if(rowcount_map_[aux] >= rows_)
				rowcount_map_[aux] = 0;
			


			query << "REPLACE INTO net_load VALUES (" <<
					rowcount_map_[aux] << "," <<
					"'" << data_seq[i].host.hostname << "'," <<
					"null" << "," <<
					"'" << data_seq[i].device << "'," <<
					data_seq[i].flags << "," <<
					data_seq[i].if_flags << "," <<
					data_seq[i].mtu << "," <<
					"'" << data_seq[i].address_str << "'," <<
					"'" << data_seq[i].subnet_str << "'," <<
					"'" << data_seq[i].hwaddress_str << "'," <<
					data_seq[i].packets_in << "," <<
					data_seq[i].packets_out << "," <<
					data_seq[i].bytes_in << "," <<
					data_seq[i].bytes_out << "," <<
					data_seq[i].bytes_total << "," <<
					data_seq[i].errors_in << "," <<
					data_seq[i].errors_out << "," <<
					data_seq[i].errors_total << "," <<
					data_seq[i].collisions << ")";
					
			query.execute();
			rowcount_map_[aux]++;
        }

    }

    retcode = netload_reader->return_loan(data_seq, info_seq);
    if (retcode != DDS_RETCODE_OK) {
        printf("return loan error %d\n", retcode);
    }
}
